package main.masterthesis.adapter;

import ie.deri.wsmx.adapter.Adapter;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.StringReader;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.soap.SOAPElement;

import org.apache.log4j.Logger;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.omwg.ontology.Concept;
import org.omwg.ontology.Instance;
import org.wsmo.common.IRI;
import org.wsmo.execution.common.nonwsmo.WSMLDocument;
import org.wsmo.execution.common.nonwsmo.grounding.EndpointGrounding;
import org.xml.sax.InputSource;

import com.isoco.dip.adapter.XMLTranslator;

/**
 * Adapter used when communicating with the Abundant Technologies Barnes & Noble
 * book price search web service. Contains methods for lowering and lifting to
 * and from WSML and XML.
 * 
 * @author yvind Skyten
 */

public class BarnesNobleAdapter extends Adapter {
	protected static Logger logger = Logger.getLogger(BarnesNobleAdapter.class);

	public BarnesNobleAdapter(String id) {
		super(id);
	}

	public BarnesNobleAdapter() {
		super();
	}

	public SOAPElement getHeader(Instance instance) {
		return null;
	}

	/**
	 * This method lowers the information in the goal WSML to an XML that can be
	 * sent to the Web service.
	 * 
	 * @param instance
	 *            the Instance created from the goal-input in the WSML.
	 * @return the XML to be sent to the Web service
	 */
	public org.w3c.dom.Document getXML(Instance instance) {
		logger.debug("enter in getXML. Instance: " + instance);
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			factory.setNamespaceAware(true);
			DocumentBuilder builder = factory.newDocumentBuilder();

			Set concepts = instance.listConcepts();
			logger.debug("Concepts set:" + concepts);
			LinkedHashMap attributes = (LinkedHashMap) instance
					.listAttributeValues();
			Set names = attributes.keySet();
			Iterator iterator = names.iterator();

			if (concepts.size() != 1) {
				throw new RuntimeException(
						"The instance has more than one concept!");
			}
			Iterator iter = concepts.iterator();
			Concept concept = (Concept) iter.next();

			String iristring = concept.getIdentifier().toString();
			logger.debug("The instance belongs to: " + iristring);

			String sDoc = "";

			// a flag to check if the iri has been matched
			boolean validInstanceFound = false;

			if (iristring
					.equalsIgnoreCase("http://www.ntnu.no/ontologies/bnpriceOntology#getBNQuote")) {

				validInstanceFound = true;
				sDoc += "<GetBNQuote xmlns=\"http://www.abundanttech.com/webservices/BNPrice\">"
						+ "\n";

				IRI att = (IRI) iterator.next();
				Set values = (Set) attributes.get((Object) att);
				String sISBN = values.iterator().next().toString();
				if (att.toString().equalsIgnoreCase(
						"http://www.ntnu.no/ontologies/bnpriceOntology#sISBN")) {
					sDoc += "<sISBN>" + sISBN + "</sISBN> \n";
				}
				sDoc += "</GetBNQuote>";
			}

			// the iri has not been matched
			if (validInstanceFound == false) {
				throw new RuntimeException("The type of the instance: "
						+ iristring + " was not recognized. ");
			}

			InputSource is = new InputSource(new StringReader(sDoc));
			org.w3c.dom.Document doc = builder.parse(is);

			ByteArrayOutputStream stream = new ByteArrayOutputStream();
			OutputFormat format = new OutputFormat(doc);
			format.setIndenting(true);
			XMLSerializer serializer = new XMLSerializer(stream, format);
			serializer.serialize(doc);

			return doc;

		} catch (Exception e) {
			logger.error("Error in processing document");
			return null;
		}
	}

	/**
	 * This method lifts the incoming XML to a WSML-representation of the
	 * information in the XML. Uses a XMLTranslator with the XML and a xslt file
	 * to do the transformation from XML to WSML.
	 * 
	 * @param document
	 *            The incoming XML
	 * @param endpoint
	 *            The EndpointGrounding (not used here)
	 * @return a WSMLDocument created from the information in the XML
	 */
	public WSMLDocument getWSML(String document, EndpointGrounding endpoint) {
		WSMLDocument wsmlDocument = new WSMLDocument("");
		String initialXML = document;
		String translatedXML;
		String xslFileName = "barnesNoble.xsl";
		XMLTranslator translator = new XMLTranslator();

		InputStream xsltFile = null;
		try {
			// For some reason the reference to the file must include the folder
			// dist for the file to be found during unittests:
			// "dist/resources/resourcemanager/masterthesis/xslt/"
			xsltFile = new FileInputStream("resources" + File.separator
					+ "resourcemanager" + File.separator + "masterthesis"
					+ File.separator + "xslt" + File.separator + xslFileName);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		// Replaces the result string "Price Not Available" to avoid parsing
		// error.
		initialXML = initialXML.replace("Price Not Available",
				"PriceNotFoundError");

		translatedXML = translator.doTransformation(initialXML, xsltFile);

		wsmlDocument.setContent(translatedXML);

		return wsmlDocument;
	}

}
