package ie.deri.webservices;

import ie.deri.wsmx.commons.Helper;
import ie.deri.wsmx.executionsemantic.ExecutionSemanticsFinalResponse;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.omwg.ontology.Instance;
import org.omwg.ontology.Value;
import org.wsmo.common.Entity;
import org.wsmo.factory.Factory;
import org.wsmo.factory.WsmoFactory;

/**
 * This class defines the entry point for searching for book prices based on a
 * ISBN.
 * 
 * @author yvind Skyten
 * 
 */
public class BookPriceSearch {

	/**
	 * This method defines the operation getBookPrice. The Method creates a goal
	 * WSML with an instance, where the value from the ISBN string in the input
	 * is used. The goal is sent to the server, and the result is sent back to
	 * the requester.
	 * 
	 * @param sISBN
	 *            the ISBN of a book.
	 * @return a String with the price of the book that was searched for
	 */
	public String getBookPrice(String sISBN) {
		String wsmlMessageGoal = "wsmlVariant _\"http://www.wsmo.org/wsml/wsml-syntax/wsml-flight\" "
				+ "namespace { _\"http://www.ntnu.no/ontologies/BarnesNobleGoal#\", "
				+ "dc _\"http://purl.org/dc/elements/1.1/\", "
				+ "xsd _\"http://www.w3.org/2001/XMLSchema#\", "
				+ "bnPrice _\"http://www.ntnu.no/ontologies/bnpriceOntology#\"} "
				+ "goal _\"http://www.ntnu.no/ontologies/BarnesNobleGoal\" "
				+ "nonFunctionalProperties "
				+ "dc#title hasValue \"Barnes & Noble price search goal with instance\" "
				+ "dc#format hasValue \"text/plain\" "
				+ "dc#type hasValue _\"http://www.wsmo.org/2004/d2#goals\" "
				+ "dc#language hasValue \"en-US\" "
				+ "endNonFunctionalProperties "
				+ "importsOntology {_\"http://www.ntnu.no/ontologies/bnpriceOntology#barnesNoblePriceOntology\"} "
				+ "capability _\"http://www.ntnu.no/goals/BarnesNobleGoal#capability\" "
				+ "importsOntology _\"http://www.ntnu.no/ontologies/bnpriceOntology#barnesNoblePriceOntology\" "
				+ "sharedVariables ?request "
				+ "precondition bnPricePrecon "
				+ "nonFunctionalProperties "
				+ "dc#description hasValue \"The input has to be a getBNQuote\" "
				+ "endNonFunctionalProperties "
				+ "definedBy "
				+ "?request memberOf bnPrice#GetBNQuote. "
				+ "postcondition bnPricePostcon "
				+ "nonFunctionalProperties "
				+ "dc#description hasValue \"The output is a GetBNQuoteResult\" "
				+ "endNonFunctionalProperties "
				+ "definedBy "
				+ "?response memberOf bnPrice#GetBNQuoteResponse. "
				+ "interface _\"http://www.ntnu.no/goals/BarnesNobleGoal#interface\" "
				+ "importsOntology _\"http://www.ntnu.no/ontologies/bnpriceOntology#barnesNoblePriceOntology\" "
				+ "choreography barnesNobleChoreography "
				+ "stateSignature bnStateSignature "
				+ "in "
				+ "concept bnPrice#GetBNQuote withGrounding _\"http://www.abundanttech.com/webservices/bnprice/bnprice.wsdl#wsdl.interfaceMessageReference(bnprice/GetBNQuote/in0)\" "
				+ "out "
				+ "concept bnPrice#GetBNQuoteResponse withGrounding _\"http://www.abundanttech.com/webservices/bnprice/bnprice.wsdl#wsdl.interfaceMessageReference(bnprice/GetBNQuoteResponse/out0)\" "
				+ "transitionRules _\"http://www.ntnu.no/ontologies/BarnesNobleGoal#transitionRules\" "
				+ "forall {?request} with "
				+ "(?request memberOf bnPrice#GetBNQuote "
				+ ") do "
				+ "add(_#1 memberOf bnPrice#GetBNQuoteResponse) "
				+ "endForall "
				+ "ontology _\"http://www.ntnu.no/ontologies/getBNPriceOntology\" "
				+ "importsOntology _\"http://www.ntnu.no/ontologies/bnpriceOntology#barnesNoblePriceOntology\" "
				+ "instance "
				+ "newBNGoal memberOf bnPrice#GetBNQuote "
				+ "bnPrice#sISBN hasValue \"" + sISBN + "\"";
		System.out.println("Getting Servlet Context");

		// get reference to MBeanServer
		MBeanServer mBeanServer = (MBeanServer) Helper
				.getAttribute("MBeanServer");

		try {
			ObjectName commManagerName = new ObjectName(
					"components:name=CommunicationManager");
			//		
			// check if registered
			boolean flag = mBeanServer.isRegistered(commManagerName);
			if (!flag) {
				return "error";
			}

			Object returnData = mBeanServer.invoke(commManagerName,
					"achieveGoalFullResponse",
					new Object[] { wsmlMessageGoal },
					new String[] { "java.lang.String" });
			ExecutionSemanticsFinalResponse esResponse = (ExecutionSemanticsFinalResponse) returnData;

			WsmoFactory wsmoFactory = Factory.createWsmoFactory(null);

			System.out.println("Recieved mesesages: "
					+ esResponse.getReceivedMessages());
			if (esResponse.getReceivedMessages().size() != 1) {
				esResponse.getExecutionSemantic().cleanUp();
				return "PriceNotFoundError";
			}
			List<Entity> instancesResponse = Helper
					.getInstancesOfConcept(new ArrayList(esResponse
							.getReceivedMessages()),
							"http://www.ntnu.no/ontologies/bnpriceOntology#GetBNQuoteResponse");
			Set attrValues = ((Instance) instancesResponse.get(0))
					.listAttributeValues(wsmoFactory
							.createIRI("http://www.ntnu.no/ontologies/bnpriceOntology#GetBNQuoteResult"));
			Value v = ((Value) attrValues.toArray()[0]);

			// clean up after communication
			esResponse.getExecutionSemantic().cleanUp();

			return v.toString();

		} catch (Exception e) {
			e.printStackTrace();
			return "Error";
		}
	}

}
